from abstractioutils.exceptions.http_exception import HTTPException


class CloudRunException(HTTPException):
    def __init__(self, status_code: int, message: str):
        super().__init__(status_code, message)



from typing import Optional, Dict
from pydantic import BaseModel


class ClusterDTO(BaseModel):
    name: Optional[str]
    image_url: Optional[str]
    vcpus: Optional[int]
    memory: Optional[int]
    user_id: Optional[str]
    port: Optional[int]
    has_ip: Optional[bool]
    type: Optional[str]
    region: Optional[str]
    project: Optional[str]

    id: Optional[str]
    status: Optional[str]
    creation_date: Optional[str]
    env_variables: Optional[Dict[str, str]]
    endpoint: Optional[str]
    ip_address: Optional[str]
    subnetwork: Optional[str]
    error_message: Optional[str]
    username: Optional[str]
    password: Optional[str]



import json
import abc

from google.oauth2 import service_account


class Base(metaclass=abc.ABCMeta):
    def __init__(self, service_account_info: str):
        self.__service_account_info = service_account_info

    def __get_credentials(self, scopes: list) -> service_account.Credentials:
        credentials = service_account.Credentials.from_service_account_info(json.loads(self.__service_account_info))
        credentials = credentials.with_scopes([single_scope for single_scope in scopes])
        return credentials

    def _get_credentials(self) -> service_account.Credentials:
        return self.__get_credentials(scopes=['https://www.googleapis.com/auth/cloud-platform'])



from datetime import datetime

from abstractioutils.providers.aws.base import Base
from abstractioutils.exceptions.aws.dynamodb_exception import DynamoDBException


class DynamoDB(Base):
    def __init__(self):
        super().__init__('dynamodb')

    def get_item(
        self,
        table_name: str,
        key: dict
    ) -> dict:
        try:
            return self._get_client().get_item(
                TableName=table_name,
                Key=key
            )
        except Exception as exc:
            print(f"{datetime.now()} - Error while getting item from the DynamoDB table - {exc}")
            raise DynamoDBException(status_code=500, message=exc.__str__())

    def delete_item(
        self,
        table_name: str,
        key: dict
    ) -> dict:
        try:
            return self._get_client().delete_item(
                TableName=table_name,
                Key=key,
                ReturnValues='ALL_OLD'
            )
        except Exception as exc:
            print(f"{datetime.now()} - Error while deleting item from the DynamoDB table - {exc}")
            raise DynamoDBException(status_code=500, message=exc.__str__())

    def scan(
        self,
        table_name: str,
        attributes_to_get: list
    ) -> dict:
        try:
            return self._get_client().scan(
                TableName=table_name,
                AttributesToGet=attributes_to_get,
                Select='SPECIFIC_ATTRIBUTES'
            )
        except Exception as exc:
            print(f"{datetime.now()} - Error while scanning items from the DynamoDB table - {exc}")
            raise DynamoDBException(status_code=500, message=exc.__str__())

    def query(
        self,
        table_name: str,
        index_name: str,
        key_condition_expression: str,
        expression_attribute_values: dict
    ) -> dict:
        try:
            return self._get_client().query(
                TableName=table_name,
                IndexName=index_name,
                KeyConditionExpression=key_condition_expression,
                ExpressionAttributeValues=expression_attribute_values
            )
        except Exception as exc:
            print(f"{datetime.now()} - Error while querying the DynamoDB table - {exc}")
            raise DynamoDBException(status_code=500, message=exc.__str__())

    def put_item(
        self,
        table_name: str,
        new_item: dict
    ) -> None:
        try:
            self._get_client().put_item(
                TableName=table_name,
                Item=new_item
            )
        except Exception as exc:
            print(f"{datetime.now()} - Error while creating a new DynamoDB item - {exc}")
            raise DynamoDBException(status_code=500, message=exc.__str__())

    def update_item(
        self,
        table_name: str,
        key: dict,
        expression_attribute_names: dict,
        expression_attribute_values: dict,
        update_expression: str
    ) -> dict:
        try:
            return self._get_client().update_item(
                TableName=table_name,
                Key=key,
                ExpressionAttributeNames=expression_attribute_names,
                ExpressionAttributeValues=expression_attribute_values,
                UpdateExpression=update_expression,
                ReturnValues='UPDATED_NEW'
            )
        except Exception as exc:
            print(f"{datetime.now()} - Error while updating a DynamoDB item - {exc}")
            raise DynamoDBException(status_code=500, message=exc.__str__())

    def delete_attribute(
        self,
        table_name: str,
        key: dict,
        expression_attribute_names: dict,
        update_expression: str
    ) -> dict:
        try:
            return self._get_client().update_item(
                TableName=table_name,
                Key=key,
                ExpressionAttributeNames=expression_attribute_names,
                UpdateExpression=update_expression
            )
        except Exception as exc:
            print(f"{datetime.now()} - Error while deleting attribute from a DynamoDB item - {exc}")
            raise DynamoDBException(status_code=500, message=exc.__str__())



import os

from time import sleep
from datetime import datetime

from abstractioutils.providers.aws.dynamodb import DynamoDB
from abstractioutils.providers.aws.sqs import SQS
from abstractioutils.providers.aws.sns import SNS
from abstractioutils.providers.aws.ssm import SSM

from abstractioutils.providers.gcp.cloud_run import CloudRun
from abstractioutils.providers.gcp.compute_engine import ComputeEngine

from abstractioutils.exceptions.helpers_exception import HelpersException

from abstractioutils.dto.cluster_dto import ClusterDTO


class CommonOperations(object):
    def __init__(self):
        self._dynamodb = DynamoDB()
        self._sqs = SQS()
        self._sns = SNS()
        self._ssm = SSM()

        self._cloud_run = None
        self._compute_engine = None

    @staticmethod
    def _lint_cluster(dynamo_obj: dict) -> ClusterDTO:
        def lint_env_variables() -> dict:
            return_value = dict()
            if dynamo_obj.get('env_variables') is not None:
                for single_item in dynamo_obj['env_variables']['SS']:
                    split_item = single_item.split('=')
                    return_value[split_item[0]] = split_item[1]
            return return_value

        return ClusterDTO.parse_obj({
            'id': dynamo_obj['id']['S'],
            'name': dynamo_obj['id']['S'],
            'image_url': dynamo_obj['image_url']['S'],
            'vcpus': dynamo_obj['vcpus']['N'],
            'memory': dynamo_obj['memory']['N'],
            'user_id': dynamo_obj['user_id']['S'],
            'port': dynamo_obj['port']['N'],
            'has_ip': dynamo_obj['has_ip']['BOOL'],
            'type': dynamo_obj['type']['S'],
            'region': dynamo_obj['region']['S'],
            'status': dynamo_obj['status']['S'],
            'creation_date': dynamo_obj['creation_date']['S'],
            'env_variables': lint_env_variables(),
            'project': dynamo_obj['gcp_project']['S'] if dynamo_obj.get('gcp_project') else None,
            'endpoint': dynamo_obj['endpoint']['S'] if dynamo_obj.get('endpoint') else None,
            'ip_address': dynamo_obj['ip_address']['S'] if dynamo_obj.get('ip_address') else None,
            'error_message': dynamo_obj['error_message']['S'] if dynamo_obj.get('error_message') else None,
            'username': dynamo_obj['username']['S'] if dynamo_obj.get('username') else None,
            'password': dynamo_obj['password']['S'] if dynamo_obj.get('password') else None
        })

    @staticmethod
    def _lint_environment_variables(env_variables: dict) -> list:
        return [f"{key}={env_variables[key]}" for key in env_variables.keys()]

    def _loop_over_cloud_run_service(
        self,
        cluster_id: str,
        cloud_run_name: str,
        region: str,
        has_ip: bool,
        static_ip: str
    ) -> bool:
        print(f"{datetime.now()} - Looping over the Cloud Run service {cloud_run_name}...")

        count = 0
        run_info = None
        while count < int(os.environ.get("CLOUD_RUN_RETRIES")):
            print(f"{datetime.now()} - Attempt number {count}")
            run_info = self._cloud_run.get_service(name=cloud_run_name, region=region)
            if 'conditions' in run_info['status']:
                for single_condition in run_info['status']['conditions']:
                    if single_condition['type'] == 'Ready' and single_condition['status'] == 'True':
                        print(f"{datetime.now()} - Updating the cluster status...")
                        if has_ip:
                            self.update_cluster_information(
                                cluster_id=cluster_id,
                                expression_attribute_names={
                                    '#ip_address': 'ip_address',
                                    '#endpoint': 'endpoint',
                                    '#status': 'status'
                                },
                                expression_attribute_values={
                                    ':ip_address': {
                                        'S': static_ip
                                    },
                                    ':endpoint': {
                                        'S': run_info['status']['url']
                                    },
                                    ':status': {
                                        'S': 'COMPLETED'
                                    }
                                },
                                update_expression='SET #ip_address = :ip_address, #endpoint = :endpoint, #status = :status'
                            )
                        else:
                            self.update_cluster_information(
                                cluster_id=cluster_id,
                                expression_attribute_names={
                                    '#endpoint': 'endpoint',
                                    '#status': 'status'
                                },
                                expression_attribute_values={
                                    ':endpoint': {
                                        'S': run_info['status']['url']
                                    },
                                    ':status': {
                                        'S': 'COMPLETED'
                                    }
                                },
                                update_expression='SET #endpoint = :endpoint, #status = :status'
                            )
                        return True
            count += 1
            sleep(5)

        print(f"{datetime.now()} - Error while creating the Cloud Run service - {run_info['status']}")
        return False

    def _set_gcp_service_account(self, cluster: ClusterDTO) -> None:
        print(f"{datetime.now()} - Setting the service account...")
        service_account = self._ssm.get_parameter(
            name=f"/gcp/{cluster.project}/sa"
        )

        self._cloud_run = CloudRun(service_account_info=service_account)
        self._compute_engine = ComputeEngine(service_account_info=service_account)

    def get_cluster_information(self, cluster_id: str) -> ClusterDTO:
        print(f"{datetime.now()} - Getting the cluster {cluster_id}")
        cluster_info = self._dynamodb.get_item(
            table_name=os.environ.get('TABLE_CLUSTERS'),
            key={
                'id': {'S': cluster_id}
            }
        )

        if cluster_info.get('Item') is None:
            print(f"{datetime.now()} - No cluster found with the given id")
            raise HelpersException(status_code=404, message='No cluster found with the given id')
        return self._lint_cluster(dynamo_obj=cluster_info['Item'])

    def update_cluster_information(
        self,
        cluster_id: str,
        expression_attribute_names: dict,
        expression_attribute_values: dict,
        update_expression: str
    ) -> dict:
        return self._dynamodb.update_item(
            table_name=os.environ.get('TABLE_CLUSTERS'),
            key={
                'id': {
                    'S': cluster_id,
                }
            },
            expression_attribute_names=expression_attribute_names,
            expression_attribute_values=expression_attribute_values,
            update_expression=update_expression
        )

    def send_message_to_sqs(self, message_body: dict, queue_url: str) -> str:
        print(f"{datetime.now()} - Sending the message to SQS - {queue_url}")
        message_ack = self._sqs.send_message(
            queue_url=queue_url,
            message_body=message_body
        )

        if message_ack is None:
            print(f"{datetime.now()} - Something wrong with SQS send message")
            raise HelpersException(status_code=500, message='Something wrong with SQS send message')
        return message_ack

    def publish_message_to_sns(self, message_body: dict, topic_arn: str) -> str:
        print(f"{datetime.now()} - Publishing the failure message to SNS - {topic_arn}")
        publish_ack = self._sns.publish_message(
            topic_arn=topic_arn,
            message=message_body
        )

        if publish_ack is None:
            print(f"{datetime.now()} - Something wrong with SNS publish message")
            raise HelpersException(status_code=500, message='Something wrong with SNS publish message')
        return publish_ack



import os

from datetime import datetime
from time import sleep

from abstractioutils.providers.gcp.base import Base
from googleapiclient.discovery import build

from abstractioutils.exceptions.gcp.compute_engine_exception import ComputeEngineException


class ComputeEngine(Base):
    def __init__(self, service_account_info: str):
        super().__init__(service_account_info)

    def reserve_static_ip_address(self, project: str, name: str, region: str) -> dict:
        print(f"{datetime.now()} - Reserving static IP...")

        service = build('compute', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            return service.addresses().insert(
                project=project,
                region=region,
                body={
                    "name": name
                }
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while reserving static IP - {exc.__str__()}")
            raise ComputeEngineException(status_code=500, message='Error while reserving static IP')

    def delete_static_ip_address(self, project: str, region: str, address: str) -> dict:
        print(f"{datetime.now()} - Deleting static IP {address}...")

        service = build('compute', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            return service.addresses().delete(
                project=project,
                region=region,
                address=address
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while deleting static IP {address} - {exc.__str__()}")
            raise ComputeEngineException(status_code=500, message='Error while deleting static IP')

    def get_static_ip_address(self, project: str, name: str, region: str) -> (str, str):
        print(f"{datetime.now()} - Getting the {name} static IP...")

        service = build('compute', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            count = 0
            while count < int(os.environ.get("STATIC_ADDRESS_RETRIES")):
                static_ip = service.addresses().get(
                    project=project,
                    region=region,
                    address=name
                ).execute()

                if 'address' in static_ip and 'selfLink' in static_ip:
                    return static_ip['address'], static_ip['selfLink']
                count += 1
                sleep(5)
            raise Exception("Not able to get IP address")
        except Exception as exc:
            print(f"{datetime.now()} - Error while getting static IP - {exc.__str__()}")
            raise ComputeEngineException(status_code=500, message='Error while getting static IP')

    def create_cloud_router_with_nat(
        self,
        project: str,
        name: str,
        region: str,
        static_ip: str,
        subnetwork: str
    ) -> dict:
        print(f"{datetime.now()} - Creating the Cloud Router...")

        service = build('compute', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            return service.routers().insert(
                project=project,
                region=region,
                body={
                    "name": name,
                    "nats": [{
                        "name": name,
                        "natIpAllocateOption": "MANUAL_ONLY",
                        "natIps": [static_ip],
                        "sourceSubnetworkIpRangesToNat": "LIST_OF_SUBNETWORKS",
                        "subnetworks": [{
                            "name": f"regions/{region}/subnetworks/{subnetwork}"
                        }]
                    }],
                    "network": f"https://www.googleapis.com/compute/v1/projects/{project}/global/networks/abstractio"
                }
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while creating Cloud Router - {exc.__str__()}")
            raise ComputeEngineException(status_code=500, message='Error while creating Cloud Router')

    def delete_cloud_router(
        self,
        project: str,
        region: str,
        router_name: str
    ) -> dict:
        print(f"{datetime.now()} - Deleting the Cloud Router {router_name}...")

        service = build('compute', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            return service.routers().delete(
                project=project,
                region=region,
                router=router_name
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while deleting the cloud router {router_name} - {exc.__str__()}")
            raise ComputeEngineException(status_code=500, message='Error while creating the cloud router')

    def create_compute_engine_instance(
        self,
        project: str,
        zone: str,
        body: dict
    ) -> dict:
        service = build('compute', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            return service.instances().insert(
                project=project,
                zone=zone,
                body=body
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while creating the VM {body['name']} - {exc.__str__()}")
            raise ComputeEngineException(status_code=500, message='Error while creating the VM')

    def delete_compute_engine_virtual_machine(
        self,
        project: str,
        zone: str,
        name: str
    ) -> dict:
        print(f"{datetime.now()} - Deleting the {name} VM...")

        service = build('compute', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            return service.instances().delete(
                project=project,
                zone=zone,
                instance=name
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while deleting the VM {name} - {exc.__str__()}")
            raise ComputeEngineException(status_code=500, message='Error while deleting the VM')


from datetime import datetime

from abstractioutils.providers.gcp.base import Base
from googleapiclient.discovery import build

from abstractioutils.exceptions.gcp.cloud_run_exception import CloudRunException


class CloudRun(Base):
    def __init__(self, service_account_info: str):
        super().__init__(service_account_info)

    def list_services(self, project_id: str, region: str) -> dict:
        print(f"{datetime.now()} - Listing Cloud Run services...")

        service = build('run', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            service._baseUrl = f'https://{region}-run.googleapis.com'
            return service.namespaces().services().list(
                parent=f'namespaces/{project_id}'
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while listing Cloud Run clusters - {exc.__str__()}")
            raise CloudRunException(status_code=500, message='Error while listing Cloud Run clusters')

    def get_service(self, name: str, region: str) -> dict:
        print(f"{datetime.now()} - Getting Cloud Run service {name}...")

        service = build('run', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            service._baseUrl = f'https://{region}-run.googleapis.com'
            return service.namespaces().services().get(
                name=name
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while getting Cloud Run cluster - {exc.__str__()}")
            raise CloudRunException(status_code=500, message=f'Error while getting Cloud Run cluster {name}')

    def create_cloud_run_service(self, project_id: str, region: str, body: dict) -> dict:
        print(f"{datetime.now()} - Creating the Cloud Run service...")

        service = build('run', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            service._baseUrl = f'https://{region}-run.googleapis.com'
            return service.namespaces().services().create(
                parent=f'namespaces/{project_id}',
                body=body
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while creating Cloud Run service - {exc.__str__()}")
            raise CloudRunException(status_code=500, message='Error while creating Cloud Run service')

    def update_cloud_run_service(self, name: str, region: str, body: dict) -> dict:
        print(f"{datetime.now()} - Updating the Cloud Run service...")

        service = build('run', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            service._baseUrl = f'https://{region}-run.googleapis.com'
            return service.namespaces().services().replaceService(
                name=name,
                body=body
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while updating Cloud Run service - {exc.__str__()}")
            raise CloudRunException(status_code=500, message='Error while updating Cloud Run service')

    def delete_cloud_run_service(self, service_name: str, region: str) -> dict:
        print(f"{datetime.now()} - Deleting the {service_name} cloud run service...")

        service = build('run', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            service._baseUrl = f'https://{region}-run.googleapis.com'
            return service.namespaces().services().delete(
                name=service_name
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while deleting the service - {exc.__str__()}")
            raise CloudRunException(status_code=500, message='Error while deleting the service')

    def allow_unauthenticated_invokations(self, service_name: str) -> dict:
        print(f"{datetime.now()} - Allowing unauth calls for the {service_name} service...")

        service = build('run', 'v1', credentials=self._get_credentials(), cache_discovery=False)
        try:
            return service.projects().locations().services().setIamPolicy(
                resource=service_name,
                body={
                    "policy": {
                        "bindings": [{
                            "role": 'roles/run.invoker',
                            "members": ['allUsers']
                        }]
                    }
                }
            ).execute()
        except Exception as exc:
            print(f"{datetime.now()} - Error while allowing unauth calls - {exc.__str__()}")
            raise CloudRunException(status_code=500, message='Error while allowing unauth calls')


import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
# noinspection PyInterpreter
setup(
    name="abstractioutils",
    version="1.0.3",
    description="Common packages used in Abstractio",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/stefanocascavilla/abstractioutils",
    author="Stefano Cascavilla",
    author_email="cascavilla1@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=["boto3==1.16.63", "pydantic==1.8.1", "oauth2client==4.1.3", "google-api-python-client==1.8.0", "google-auth==1.28.1"],
)
